/*******************************************************************************
*
*                 TRIGGER THE ADC WITH AN OVERFLOW OF TIMERR 0
*                 --------------------------------------------
*
*   Filename:                                    
*   Autor:                  David Kress
*
*   Version:                1.0
*   Date:                   5. December 2010
*   Built with:             ATMEL AVR Studio 4.14 and AVR-GCC; Optimization -01
*
*   Description:            This programm is a simple example how to trigger
*                           the ADC with the compare match of Timerr 0. This
*                           way youcan realize a sampling which runs independent
*                           from your mainprogramm. You find more informations
*                           about the setup of Timer 0in the file
*                           
*                           The samplingrate is about 4Hz. A toggeling LED will
*                           indicate the samplemoments. The ADC value is used
*                           to operate a second LED. This LED will only be
*                           updated at the samplingmoments.
*                           Because the triggerfrequency is 4Hz, it is not
*                           easy to see, that the ADC-LED is always updatet
*                           at the same time as the timer-LED. Turn the
*                           potentiometer fast to see it or use an oscilloscope.
*                           
*                           
*   What you can learn:     - Sample with the ADC on a Timer 0 compare match
*                           - Setup the ADC
*                           - Use Timer 0 in CTC mode
*
*   Clock source:           - Any 1Mhz clock
*
*   Recommended             - High: 0x99
*   Fuse settings:          - Low:  0xE1
*
*
*
*   Schematic:       ___________________
*                   |      ATMEGA16     |
*          Vcc ---->|Vcc                |
*          Vcc ---->|RESET              |
*          Vcc ---->|AVcc          AREF |----
*                   |                   |
*          GND ---->|GND                |   |
*                   |                   |  ===
*          Vcc      |                   |   |
*           |       |                   |  GND
*           |       |                   |             
*          |P|      |                   |
*          |O|      |                   |
*          |T|----->| PA0  (ADC0)       |             
*          |I|      |                   |
*           |       |                   |
*           |       |               PB2 |----> DOUT     
*          GND      |          PB3 (OC0)|----> DOUT    |
*                   |                   |
*
*******************************************************************************/


#include <avr/io.h>
#include <avr/interrupt.h>

int main(void)
{

     DDRB = 0x0c;                     // Setup PB2 and PB3 as output

     OCR0 = 243;                      // Set the capturevalue
     
     TCCR0 |= (1<<CS02)|(1<<CS00)|    // Start timer0 with prescaler 1024
              (1<<WGM01)|(1<<COM00);  // CTC mode, toggle OC0 on compare match
   
     SFIOR |= (1<<ADTS0)|(1<<ADTS1);  // Set Timer 0 Compare as ADC trigger
  
     ADMUX |= (1<<REFS0);             // Set reference to AVcc

     ADCSRA |= (1<<ADEN)|(1<<ADATE)|  // Enable ADC, Enable auto Triggering
               (1<<ADPS2)|(1<<ADIE);  // Set prescaler to 16
                                      // Fadc=Fcpu/prescaler= 1000000/16=62.5kHz
                                      // Fadc should be between 50kHz and 200kHz

     sei();                           // Set the I-bit in SREG

     for(;;);                         // Endless loop;
                                      // main() will never be left

     return 0;                        // This line will never be executed

}

// Interrupt subroutine for ADC conversion complete interrupt
 ISR(ADC_vect)
{
         if(ADC >= 512)                // Compare the conversionresult with 512
             PORTB |= 0x04;            // If larger, set PB3
         else
             PORTB &= ~0x04;           // If smaller, reset PB3

         TIFR |= (1<<OCF0);            // Clear the Timer 0 interruptflag
}

